program vidttic; // VIDTitle Template Image Creator
uses
  sysutils,
  arena2_vid, arena2_gfx;

type
  TRGBQuad = packed record
    B: byte;
    G: byte;
    R: byte;
    O: byte; // Always 0
  end;
  TBMP8bitHeader = packed record
    // BITMAPFILEHEADER
    ID: packed array[1..2] of char; // 'BM'
    FileSize: longword; // DataSize + Offset
    Res1: word;
    Res2: word;
    Offset: longword; // 1078 = SizeOf(TBMPHeader) + SizeOf(Palette)
    // BITMAPINFO <CORE>
    IHSize: longword; // 40
    Width: longword;
    Height: longword;
    Planes: word; // 1
    BitCount: word; // 8
    // </CORE>
    Compress: longword;
    DataSize: longword; // Width * Height
    XPxPerMeter: longword; // dpi * 39.37ish
    YPxPerMeter: longword; // dpi * 39.37ish
    Colors: longword; // 256
    IColors: longword; // 256
    // bmiColors
    Palette: array[0..255] of TRGBQuad;
  end;

//********************************************************//

var
  Header: TVIDHeader;
  Palette: TPAL;
  BMPHeader: TBMP8bitHeader;
  FrameSize: longword;
  AudioData, ThisFrame: packed array[0..65535] of byte;
  InputFile, OutputFile: file;
  InputFN, OutputFN: string;
  T: longword = 0;
  i, YOffset, FrameDelay, AudioLength: word;
  b, PlaybackRate, BlockType, RunLength: byte;
  Z: longword;
  PalCount: longword = 0;

begin
  BMPHeader.ID := 'BM';
  BMPHeader.Res1 := 0;
  BMPHeader.Res2 := 0;
  BMPHeader.Offset := 1078; // SizeOf(TBMPHeader) + SizeOf(Palette)
  BMPHeader.IHSize := 40;
  BMPHeader.Planes := 1;
  BMPHeader.BitCount := 8;
  BMPHeader.Compress := 0;
  BMPHeader.XPxPerMeter := 2835; // 72 dpi * 39.37ish
  BMPHeader.YPxPerMeter := 2835; // 72 dpi * 39.37ish
  BMPHeader.Colors := 256;
  BMPHeader.IColors := 256;

  if (ParamCount < 1) then
   begin
    writeln('Usage: ' + ParamStr(0) + ' <Input File>');
    writeln;
    writeln('.');
    halt(1)
   end;

  InputFN := ParamStr(1);
  Filemode := 0; // should be Read - default = ReadWrite = run-time error when opening read-only files
  assign(InputFile, InputFN);
  reset(InputFile, 1);
  Filemode := 2; // back to ReadWrite - just in case leaving it as Read does something even stupider

  BlockRead(InputFile, Header, SizeOf(Header));
  writeln(Header.ID, ' header:');
  writeln('  FrameCount: ', Header.FrameCount);
  writeln('  FrameWidth: ', Header.FrameWidth);
  writeln('  FrameHeight: ', Header.FrameHeight);
  writeln('  GlobalDelay: ', Header.GlobalDelay);
  writeln('  Unknown2: ', Header.Unknown2);

  FrameSize := (Header.FrameWidth * Header.FrameHeight);

  BMPHeader.FileSize := FrameSize + BMPHeader.Offset;
  BMPHeader.Width := Header.FrameWidth;
  BMPHeader.Height := Header.FrameHeight;
  BMPHeader.DataSize := FrameSize; // Width * Height

  for i := 0 to 255 do
   begin
    BMPHeader.Palette[i].R := i;
    BMPHeader.Palette[i].G := i;
    BMPHeader.Palette[i].B := i;
    BMPHeader.Palette[i].O := 0;
   end;


  OutputFN := InputFN + '_mask_template.bmp';
  assign(OutputFile, OutputFN);
  rewrite(OutputFile, 1);
  BlockWrite(OutputFile, BMPHeader, SizeOf(BMPHeader));

  FillByte(ThisFrame, SizeOf(ThisFrame), 0);
  BlockWrite(OutputFile, ThisFrame, FrameSize);

  close(OutputFile);

  repeat
    BlockRead(InputFile, BlockType, SizeOf(BlockType)); // get block type

//************************************//
    if (BlockType = VIDBLOCK_PAL) then
     begin
      writeln(HexStr(FilePos(InputFile), 8), ': ', BlockType, ': Palette');
      BlockRead(InputFile, Palette, SizeOf(Palette));
      for i := 0 to 255 do
       begin
        BMPHeader.Palette[i].R := Palette[i].R * 4;
        BMPHeader.Palette[i].G := Palette[i].G * 4;
        BMPHeader.Palette[i].B := Palette[i].B * 4;
        BMPHeader.Palette[i].O := 0;
       end;
      inc(PalCount, 1);

      OutputFN := InputFN + '_overlay_template' + Format('%.2D', [PalCount]) + '.bmp';
      assign(OutputFile, OutputFN);
      rewrite(OutputFile, 1);
      BlockWrite(OutputFile, BMPHeader, SizeOf(BMPHeader));

      Z := FrameSize DIV 256;
      for i := 0 to 255 do
        FillByte(ThisFrame[(Z * i)], Z, i);

      for i := (Header.FrameHeight - 1) downto 0 do
        BlockWrite(OutputFile, ThisFrame[(i * Header.FrameWidth)], Header.FrameWidth);

      close(OutputFile);
     end

//************************************//
    else if (BlockType in [VIDBLOCK_V03, VIDBLOCK_V01, VIDBLOCK_V04]) then
     begin
      BlockRead(InputFile, FrameDelay, SizeOf(FrameDelay));


      if (BlockType = VIDBLOCK_V04) then
       begin
        BlockRead(InputFile, YOffset, SizeOf(YOffset));
        Z := (YOffset * Header.FrameWidth);
       end
      else
        Z := 0;

      // decode frame
      repeat
        BlockRead(InputFile, RunLength, SizeOf(RunLength));
        if (RunLength > 127) then
         begin
          if (BlockType = VIDBLOCK_V03) then
            BlockRead(InputFile, b, SizeOf(b));

          inc(Z, (RunLength - 128));
         end
        else
         begin
          BlockRead(InputFile, ThisFrame[Z], RunLength);
          inc(Z, RunLength);
         end;
      until (RunLength = 0) or (Z >= FrameSize);

      inc(T, (FrameDelay + Header.GlobalDelay));
     end

//************************************//
    else if (BlockType = VIDBLOCK_AU1) then
     begin
      BlockRead(InputFile, i, SizeOf(i));
      BlockRead(InputFile, PlaybackRate, SizeOf(PlaybackRate));
      BlockRead(InputFile, AudioLength, SizeOf(AudioLength));
      writeln(HexStr(FilePos(InputFile), 8), ': ', BlockType, ': Audio ', (1000000 DIV (256 - PlaybackRate)), 'Hz');
      BlockRead(InputFile, AudioData, AudioLength);
     end

//************************************//
    else if (BlockType = VIDBLOCK_AUD) then
     begin
      BlockRead(InputFile, AudioLength, SizeOf(AudioLength));
      BlockRead(InputFile, AudioData, AudioLength);
     end

//************************************//
    else
     begin
      if (BlockType = VIDBLOCK_EOF) then
        writeln(HexStr(FilePos(InputFile), 8), ': ', BlockType, ': EOF')
      else if (BlockType <> 0) then
        writeln(HexStr(FilePos(InputFile), 8), ': ', BlockType, ': ???');
     end;

  until eof(InputFile);

  close(InputFile);
end.
